<?php

namespace domain\User\User;

use domain\Communication\Email\Email;
use domain\User\UserProducer\UserProducerBuilder;
use domain\User\UserProducer\UserProducerRepository;
use domain\_\AbstractService;
use domain\_\ManagerInterface;

class NewsletterManager extends AbstractService implements ManagerInterface
{
    protected UserProducerRepository $userProducerRepository;
    protected UserProducerBuilder $userProducerBuilder;

    public function loadDependencies(): void
    {
        $this->userProducerRepository = $this->loadService(UserProducerRepository::class);
        $this->userProducerBuilder = $this->loadService(UserProducerBuilder::class);
    }

    public function isUserSubscribedNewsletter(User $user = null): bool
    {
        if($user) {
            $userProducer = $this->userProducerRepository->findOneUserProducer($user);
            if($userProducer && $userProducer->newsletter) {
                return true;
            }
        }

        return false;
    }

    public function manageUserNewsletter(User $user, ?bool $newsletter, ?bool $newsletterOrderTaking)
    {
        if($newsletter) {
            $this->subscribeUserNewsletter($user, Email::TYPE_NEWSLETTER);
        }
        else {
            $this->unsubscribeUserNewsletter($user, Email::TYPE_NEWSLETTER);
        }

        if($newsletterOrderTaking) {

        }
        else {

        }
    }

    public function subscribeUserNewsletter(User $user, string $type): bool
    {
        $userProducer = $this->userProducerBuilder->createUserProducerIfNotExist($user, $this->getProducerContext());

        if($type == Email::TYPE_NEWSLETTER) {
            return $this->userProducerBuilder->updateUserProducerNewsletter($userProducer, true);
        }
        elseif($type == Email::TYPE_ORDER_TAKING) {
            return $this->userProducerBuilder->updateUserProducerNewsletterOrderTaking($userProducer, true);
        }

        return false;
    }

    public function unsubscribeUserNewsletter(User $user, string $type): bool
    {
        $userProducer = $this->userProducerRepository->findOneUserProducer($user);
        if($userProducer) {
            if($type == Email::TYPE_NEWSLETTER) {
                return $this->userProducerBuilder->updateUserProducerNewsletter($userProducer, false);
            }
            elseif($type == Email::TYPE_ORDER_TAKING) {
                return $this->userProducerBuilder->updateUserProducerNewsletterOrderTaking($userProducer, false);
            }
        }

        return false;
    }
}